/********************************** (C) COPYRIGHT *******************************
 * File Name          : Main.c
 * Author             : kingsleych@wch
 * Version            : V1.0
 * Date               : 2021/07/22
 * Description 		    :ch573 adc with dma
 *******************************************************************************/

#include "stdint.h"
#include "stdbool.h"

#include "CH57x_common.h"

#define GPIO_TEST 1

volatile bool adc_dma_end = false;
volatile bool adc_dma_half_end = false;


uint16_t adc_dma_buf[2000];

void DebugInit( void ) {
    GPIOA_SetBits( GPIO_Pin_9 );
    GPIOA_ModeCfg( GPIO_Pin_8, GPIO_ModeIN_PU );
    GPIOA_ModeCfg( GPIO_Pin_9, GPIO_ModeOut_PP_5mA );
    UART1_DefInit();
}

void adc_dma_data_dump(uint16_t *data,uint16_t length,uint16_t jump_size) {
    for(uint16_t i=0;i<length;i++) {
        PRINT("%04d ",data[i*jump_size]);
    }
}

int main() {

    SetSysClock( CLK_SOURCE_PLL_60MHz );
    /* ôڵ */
    DebugInit();

    PRINT( "Start @ChipID=%02X\n", R8_CHIP_ID );

#if GPIO_TEST
    GPIOA_ResetBits( GPIO_Pin_14 );
    GPIOA_ModeCfg( GPIO_Pin_14, GPIO_ModeOut_PP_5mA );

    GPIOA_ResetBits( GPIO_Pin_13);
    GPIOA_ModeCfg( GPIO_Pin_13, GPIO_ModeOut_PP_5mA );

    GPIOA_ResetBits( GPIO_Pin_12);
    GPIOA_ModeCfg( GPIO_Pin_12, GPIO_ModeOut_PP_5mA );

    GPIOB_ResetBits( GPIO_Pin_4);
    GPIOB_ModeCfg( GPIO_Pin_4, GPIO_ModeOut_PP_5mA );
#endif




#if 1
    //PA15 half voltage connected to PA4
    GPIOA_SetBits( GPIO_Pin_15 );
    GPIOA_ModeCfg( GPIO_Pin_15, GPIO_ModeOut_PP_5mA );
    TMR0_TimerInit( FREQ_SYS /250 ); //4ms
    TMR0_ITCfg( ENABLE, TMR0_3_IT_CYC_END );
    PFIC_EnableIRQ( TMR0_IRQn );
#endif

#if 1
    //start a time for dma half interrupt
    TMR1_TimerInit( FREQ_SYS /50);//20ms=1/50s
    //stop timer
    R8_TMR1_CTRL_MOD = RB_TMR_ALL_CLEAR;

    TMR1_ITCfg( ENABLE, TMR0_3_IT_CYC_END );          // ж
    PFIC_EnableIRQ( TMR1_IRQn );

#endif

    //PA4 mapping to ADC channel 0
    GPIOA_ModeCfg( GPIO_Pin_4, GPIO_ModeIN_Floating );
    ADC_ExtSingleChSampInit( SampleFreq_3_2, ADC_PGA_0 );
    ADC_ChannelCfg( 0 );

    //adc init ....
    //adc duty = (256-R8_ADC_AUTO_CYCLE)*16*Tsys

    //75*16*1/60  = 20us
    //20us * 2000 = 40000us = 40ms
    //auto cycle
    R8_ADC_AUTO_CYCLE = 256-75;
    R16_ADC_DMA_BEG = (uint16_t )&adc_dma_buf[0];
    R16_ADC_DMA_END = (uint16_t )&adc_dma_buf[2000]; //20us * 2000 = 40ms


    PRINT("R16_ADC_DMA_BEG = %08x\r\n",R16_ADC_DMA_BEG);
    PRINT("R16_ADC_DMA_END = %08x\r\n",R16_ADC_DMA_END);
    R8_ADC_CTRL_DMA  =  RB_ADC_IE_EOC| RB_ADC_IE_DMA_END |RB_ADC_DMA_LOOP |RB_ADC_DMA_ENABLE ;

    R8_ADC_CTRL_DMA  |= RB_ADC_AUTO_EN;
    PFIC_EnableIRQ( ADC_IRQn );
    // ch573 has not dma half interrupt,here ,we start a timer
    R8_TMR1_CTRL_MOD = RB_TMR_COUNT_EN;

    while( 1 ) {
        if(true == adc_dma_end) {
            PRINT("ADC_DMA_NOW address = %d\r\n",(R16_ADC_DMA_NOW-R16_ADC_DMA_BEG)/2);
            adc_dma_end = false;
            adc_dma_data_dump(&adc_dma_buf[1000],20,50);
            PRINT("\r\n");
        }
        if(true == adc_dma_half_end) {
            PRINT("ADC_DMA_NOW address = %d\r\n",(R16_ADC_DMA_NOW-R16_ADC_DMA_BEG)/2);
            adc_dma_half_end = false;
            adc_dma_data_dump(&adc_dma_buf[0],20,50);
            PRINT("\r\n");
        }
    }
}

__attribute__((interrupt("WCH-Interrupt-fast")))
__attribute__((section(".highcode")))
void ADC_IRQHandler( void ) {		//adcжϷ
    //optional for testing
    if(RB_ADC_IF_END_ADC  & RB_ADC_IF_END_ADC   ) {
#if GPIO_TEST
        GPIOA_InverseBits( GPIO_Pin_12);
        __nop();
        GPIOA_InverseBits( GPIO_Pin_12);
#endif
    }

    //RB_ADC_IF_END_ADC should not be clear before RB_ADC_IF_DMA_END
    if(R8_ADC_DMA_IF & RB_ADC_IF_DMA_END ) {

#if GPIO_TEST
        GPIOA_InverseBits( GPIO_Pin_13);
        __nop();
        GPIOA_InverseBits( GPIO_Pin_13);
        GPIOB_SetBits( GPIO_Pin_4 );
#endif
        //adc dma end , we can read the second half part of dma buffer
        //0-1249    for the first half part
        //1250-2499 for the second half part
        adc_dma_end = true;

        // ch573 has not dma half interrupt,here ,we start a timer
        R8_TMR1_CTRL_MOD = RB_TMR_COUNT_EN;
    }

    //clear all adc dma interrupt flag
    R8_ADC_DMA_IF |=R8_ADC_DMA_IF;
}


__attribute__((interrupt("WCH-Interrupt-fast")))
__attribute__((section(".highcode")))
void TMR0_IRQHandler( void )
{
  if ( TMR0_GetITFlag( TMR0_3_IT_CYC_END ) )
  {
    TMR0_ClearITFlag( TMR0_3_IT_CYC_END );
    GPIOA_InverseBits( GPIO_Pin_15 );
  }
}

__attribute__((interrupt("WCH-Interrupt-fast")))
__attribute__((section(".highcode")))
void TMR1_IRQHandler( void )
{
  if ( TMR1_GetITFlag( TMR0_3_IT_CYC_END ) )
  {
    TMR1_ClearITFlag( TMR0_3_IT_CYC_END );
#if GPIO_TEST
    GPIOB_ResetBits( GPIO_Pin_4 );
#endif
    //stop timer
    R8_TMR1_CTRL_MOD = RB_TMR_ALL_CLEAR;
    adc_dma_half_end = true;
  }
}



